@@ -194,80 +194,6 @@ rInvWishart <- function(n, df, Sigma) {
 }
 
 
-#' Random Pseudo Wishart Distributed Matrices
-#'
-#' @description Generate n random matrices, distributed according
-#'     to the pseudo Wishart distribution with parameters \code{Sigma} and
-#'     \code{df}, \eqn{W_p(\Sigma, df)}{W_p(Sigma, df)}, with sample size
-#'     \code{df} less than the dimension \code{p}.
-#'
-#'     Let \eqn{X_i}, \eqn{i = 1, 2, ..., df} be \code{df}
-#'     observations of a multivariate normal distribution with mean 0 and
-#'     covariance \code{Sigma}. Then \eqn{\sum X_i X_i'} is distributed as a pseudo
-#'     Wishart \eqn{W_p(\Sigma, df)}{W_p(Sigma, df)}. Sometimes this is called a
-#'     singular Wishart distribution, however, that can be confused with the case
-#'     where \eqn{\Sigma}{Sigma} itself is singular. This can generate samples for
-#'     positive semi-definite \eqn{\Sigma}{Sigma}, however, a function dedicated to
-#'     generating singular normal random distributions or singular pseudo Wishart
-#'     distributions should be used if that is desired.
-#'
-#' @param n integer sample size.
-#' @param df integer parameter, "degrees of freedom", should be less than the
-#'    dimension of \code{p}
-#' @param Sigma positive semi-definite \eqn{p \times p}{(p * p)} "scale" matrix, the
-#' matrix
-#'    parameter of the distribution.
-#'
-#' @return a numeric array, say \code{R}, of dimension \eqn{p \times p \times n}{p * p * n},
-#'     where each \code{R[,,i]} is a realization of the pseudo Wishart distribution
-#'     \eqn{W_p(Sigma, df)}.
-#'
-#' @seealso \code{\link{rWishart}}, \code{\link{rInvWishart}},
-#'     and \code{\link{rGenInvWishart}}
-#'
-#' @references
-#' Diaz-Garcia, Jose A, Ramon Gutierrez Jaimez, and Kanti V Mardia. 1997.
-#' “Wishart and Pseudo-Wishart Distributions and Some Applications to Shape Theory.”
-#' Journal of Multivariate Analysis 63 (1): 73–87. \doi{10.1006/jmva.1997.1689}.
-#'
-#' Uhlig, Harald. On Singular Wishart and Singular Multivariate Beta Distributions.
-#' Ann. Statist. 22 (1994), no. 1, 395--405. \doi{10.1214/aos/1176325375}.
-#'
-#' @export
-#'
-#' @examples
-#' set.seed(20181227)
-#' A<-rPseudoWishart(1,4,5.0*diag(5))[,,1]
-#' # A should be singular
-#' eigen(A)$values
-#'
-rPseudoWishart <- function(n, df, Sigma) {
-  tol = 1e-06
-  p <- ncol(Sigma)
-  if (!all(dim(Sigma) == c(p, p)))
-    stop("Sigma must be square.")
-  if (df > p - 1) {
-    warning("df > dimension of Sigma - 1, using rWishart.")
-    return(stats::rWishart(n, df, Sigma))
-  }
-  if (!(df == round(df))) stop("df needs to be a whole number.")
-  if (df < 1) stop("df needs to be greater than 1.")
-  eigens <- eigen(Sigma, symmetric = TRUE)
-  ev <- eigens$values
-  if (!all(ev >= -tol * abs(ev[1L])))
-    stop("'Sigma' is not positive semidefinite.")
-  # Note: this can deal with 0 eigenvalues, so it could be singular.
-  # It cannot, however, deal with <0 eigenvalues. If Sigma is singular,
-  # you really should directly deal with it.
-  sqrtmatrix <- eigens$vectors %*% diag(sqrt(pmax(ev, 0)), p)
-  X <- array(stats::rnorm(p*df*n), dim = c(df, p, n))
-  Xresult <- array(0, dim = c(p, p, n))
-  for (i in 1:n) Xresult[,,i] = tcrossprod(tcrossprod(sqrtmatrix, X[,,i]))
-
-  (Xresult)
-}
-
-
 #' Random Generalized Inverse Wishart Distributed Matrices
 #'
 #' @description Generate n random matrices, distributed according
@@ -332,32 +258,27 @@ rPseudoWishart <- function(n, df, Sigma) {
 #'
 #' @examples
 #' set.seed(20181228)
-#' A<-rGenInvWishart(1,4,5.0*diag(5))[,,1]
+#' A<-rGenInvWishart(1, 4, 5.0*diag(5))[,,1]
+#' A
 #' # A should be singular
 #' eigen(A)$values
+#' set.seed(20181228)
+#' B <- rPseudoWishart(1, 4, 5.0*diag(5))[,,1]
+#' 
+#' # A should be a Moore-Penrose pseudo-inverse of B
+#' B
+#' # this should be equal to B
+#' B %*% A %*% B
+#' # this should be equal to A
+#' A %*% B %*% A
+#' 
 #'
 rGenInvWishart <- function(n, df, Sigma) {
   tol = 1e-06
+  Sigma <- as.matrix(Sigma)
+  dims = dim(Sigma)
   p <- ncol(Sigma)
-  if (!all(dim(Sigma) == c(p, p)))
-    stop("Sigma must be square.")
-  if (df > p - 1) {
-    warning("df > dimension of Sigma - 1, using rWishart.")
-    return(stats::rWishart(n, df, Sigma))
-  }
-  if (!(df == round(df))) stop("df needs to be a whole number.")
-  if (df < 1) stop("df needs to be greater than 1.")
-  eigens <- eigen(Sigma, symmetric = TRUE)
-  ev <- eigens$values
-  if (!all(ev >= -tol * abs(ev[1L])))
-    stop("'Sigma' is not positive semidefinite.")
-  # Note: this can deal with 0 eigenvalues, so it could be singular.
-  # This cannot, however, deal with <0 eigenvalues. If Sigma is singular,
-  # you really should directly deal with it.
-  sqrtmatrix <- eigens$vectors %*% diag(sqrt(pmax(ev, 0)), p)
-  X <- array(stats::rnorm(p*df*n), dim = c(df, p, n))
-  Xresult <- array(0, dim = c(p, p, n))
-  for (i in 1:n) Xresult[,,i] = tcrossprod(tcrossprod(sqrtmatrix, X[,,i]))
+  Xresult <- rPseudoWishart(n, df, Sigma)
   for (i in 1:n) {
     tmpX = Xresult[,,i]
     svdX <- svd(tmpX)
@@ -369,6 +290,65 @@ rGenInvWishart <- function(n, df, Sigma) {
 }
 
 
+
+
+#' Random Pseudo Wishart Distributed Matrices
+#'
+#' @description Generate n random matrices, distributed according
+#'     to the pseudo Wishart distribution with parameters \code{Sigma} and
+#'     \code{df}, \eqn{W_p(\Sigma, df)}{W_p(Sigma, df)}, with sample size
+#'     \code{df} less than the dimension \code{p}.
+#'
+#'     Let \eqn{X_i}, \eqn{i = 1, 2, ..., df} be \code{df}
+#'     observations of a multivariate normal distribution with mean 0 and
+#'     covariance \code{Sigma}. Then \eqn{\sum X_i X_i'} is distributed as a pseudo
+#'     Wishart \eqn{W_p(\Sigma, df)}{W_p(Sigma, df)}. Sometimes this is called a
+#'     singular Wishart distribution, however, that can be confused with the case
+#'     where \eqn{\Sigma}{Sigma} itself is singular. If cases with a singular
+#'     \eqn{\Sigma}{Sigma} are desired, this function cannot provide them.
+#'
+#' @param n integer sample size.
+#' @param df integer parameter, "degrees of freedom", should be less than the
+#'    dimension of \code{p}
+#' @param Sigma positive definite \eqn{p \times p}{(p * p)} "scale" matrix, the
+#'    matrix parameter of the distribution.
+#'
+#' @return a numeric array, say \code{R}, of dimension \eqn{p \times p \times n}{p * p * n},
+#'     where each \code{R[,,i]} is a realization of the pseudo Wishart distribution
+#'     \eqn{W_p(Sigma, df)}.
+#'
+#' @seealso \code{\link{rWishart}}, \code{\link{rInvWishart}},
+#'     and \code{\link{rGenInvWishart}}
+#'
+#' @references
+#' Diaz-Garcia, Jose A, Ramon Gutierrez Jaimez, and Kanti V Mardia. 1997.
+#' “Wishart and Pseudo-Wishart Distributions and Some Applications to Shape Theory.”
+#' Journal of Multivariate Analysis 63 (1): 73–87. \doi{10.1006/jmva.1997.1689}.
+#'
+#' Uhlig, Harald. On Singular Wishart and Singular Multivariate Beta Distributions.
+#' Ann. Statist. 22 (1994), no. 1, 395--405. \doi{10.1214/aos/1176325375}.
+#'
+#' @export
+#'
+#' @examples
+#' set.seed(20181227)
+#' A<-rPseudoWishart(1,4,5.0*diag(5))[,,1]
+#' # A should be singular
+#' eigen(A)$values
+#'
+rPseudoWishart <- function(n, df, Sigma) {
+  Sigma <- as.matrix(Sigma)
+  dims = dim(Sigma)
+  p <- ncol(Sigma)
+  if (df > p - 1) {
+    warning("df > dimension of Sigma - 1, using rWishart.")
+    return(stats::rWishart(n, df, Sigma))
+  }
+  if (!(df == round(df))) stop("df needs to be a whole number.")
+  if (df < 1) stop("df needs to be greater than 1.")
+  .Call("C_rPseudoWishart", n, df, Sigma, PACKAGE = "CholWishart")
+}
+
 .onUnload <- function(libpath) {
   library.dynam.unload("CholWishart", libpath)
 }
